@@ -25,6 +25,8 @@ Atom a_XROOTPMAP_ID;
 /* old WM spec */
 Atom a_WM_STATE;
 Atom a_WM_CLASS;
+Atom a_WM_DELETE_WINDOW;
+Atom a_WM_PROTOCOLS;
 
 /* new NET spec */
 Atom a_NET_WORKAREA;
@@ -34,6 +36,7 @@ Atom a_NET_NUMBER_OF_DESKTOPS;
 Atom a_NET_CURRENT_DESKTOP;
 Atom a_NET_DESKTOP_NAMES;
 Atom a_NET_ACTIVE_WINDOW;
+Atom a_NET_CLOSE_WINDOW;
 Atom a_NET_WM_DESKTOP;
 Atom a_NET_WM_STATE;
 Atom a_NET_WM_STATE_SKIP_TASKBAR;
@@ -54,6 +57,7 @@ Atom a_NET_WM_DESKTOP;
 Atom a_NET_WM_NAME;
 Atom a_NET_WM_STRUT;
 Atom a_NET_WM_STRUT_PARTIAL;
+Atom a_NET_WM_ICON;
 Atom a_KDE_NET_WM_SYSTEM_TRAY_WINDOW_FOR;
 
 pair allign_pair[] = {
@@ -128,11 +132,9 @@ get_line(FILE *fp, line *s)
     gchar *tmp, *tmp2;
 
     ENTER;
-    if (!fp) {
-        s->type = LINE_NONE;
-        RET(s->type);
-    }
     s->type = LINE_NONE;
+    if (!fp)       
+        RET(s->type);
     while (fgets(s->str, s->len, fp)) {
         g_strstrip(s->str);
      
@@ -209,6 +211,8 @@ void resolve_atoms()
     a_XROOTPMAP_ID               = XInternAtom(GDK_DISPLAY(), "_XROOTPMAP_ID", False);
     a_WM_STATE                   = XInternAtom(GDK_DISPLAY(), "WM_STATE", False);
     a_WM_CLASS                   = XInternAtom(GDK_DISPLAY(), "WM_CLASS", False);
+    a_WM_DELETE_WINDOW           = XInternAtom(GDK_DISPLAY(), "WM_DELETE_WINDOW", False);
+    a_WM_PROTOCOLS               = XInternAtom(GDK_DISPLAY(), "WM_PROTOCOLS", False);
     a_NET_WORKAREA               = XInternAtom(GDK_DISPLAY(), "_NET_WORKAREA", False);    
     a_NET_CLIENT_LIST            = XInternAtom(GDK_DISPLAY(), "_NET_CLIENT_LIST", False);
     a_NET_CLIENT_LIST_STACKING   = XInternAtom(GDK_DISPLAY(), "_NET_CLIENT_LIST_STACKING", False);
@@ -237,6 +241,7 @@ void resolve_atoms()
     a_NET_WM_NAME                = XInternAtom(GDK_DISPLAY(), "_NET_WM_NAME", False);
     a_NET_WM_STRUT               = XInternAtom(GDK_DISPLAY(), "_NET_WM_STRUT", False);
     a_NET_WM_STRUT_PARTIAL       = XInternAtom(GDK_DISPLAY(), "_NET_WM_STRUT_PARTIAL", False);
+    a_NET_WM_ICON                = XInternAtom(GDK_DISPLAY(), "_NET_WM_ICON", False);
     a_KDE_NET_WM_SYSTEM_TRAY_WINDOW_FOR
                                  = XInternAtom(GDK_DISPLAY(), "_KDE_NET_WM_SYSTEM_TRAY_WINDOW_FOR", False);
                                 
@@ -263,6 +268,20 @@ Xclimsg(Window win, long type, long l0, long l1, long l2, long l3, long l4)
           (XEvent *) & xev);
 }
 
+void
+Xclimsgwm(Window win, Atom type, Atom arg)
+{
+    XClientMessageEvent xev;
+
+    xev.type = ClientMessage;
+    xev.window = win;
+    xev.message_type = type;
+    xev.format = 32;
+    xev.data.l[0] = arg;
+    xev.data.l[1] = GDK_CURRENT_TIME;
+    XSendEvent(GDK_DISPLAY(), win, False, 0L, (XEvent *) &xev);
+}
+
 void *
 get_xaproperty (Window win, Atom prop, Atom type, int *nitems)
 {
@@ -587,7 +606,7 @@ expand_tilda(gchar *file)
 
 
 
-
+#if 0
 Window
 Select_Window(Display *dpy)
 {
@@ -633,7 +652,7 @@ Select_Window(Display *dpy)
     XUngrabPointer(dpy, CurrentTime);      /* Done with pointer */
     RET(target_win);
 }
-
+#endif
 
 /*
  * SuxPanel version 0.1
@@ -647,39 +666,61 @@ Select_Window(Display *dpy)
  *
  */
 
+GdkPixbuf *
+gdk_pixbuf_scale_ratio(GdkPixbuf *p, int width, int height, GdkInterpType itype, gboolean keep_ratio)
+{
+    gfloat w, h, rw, rh;
+
+    if (keep_ratio) {
+        w = gdk_pixbuf_get_width(p);
+        h = gdk_pixbuf_get_height(p);
+        rw = w / width;
+        rh = h / height;
+        if (rw > rh)
+            height = h / rw;
+        else
+            width =  w / rh;
+    }
+    return  gdk_pixbuf_scale_simple(p, width, height, itype);
+
+}
+
 
 GtkWidget *
 gtk_image_new_from_file_scaled(const gchar *file, gint width,
-      gint height)
+      gint height, gboolean keep_ratio)
 {
     GtkWidget *img;
+    GdkPixbuf *pb, *pb_scaled;
+    gfloat w, h, rw, rh;
 
     ENTER;
-    if (g_file_test(file, G_FILE_TEST_EXISTS)) {		
-        GError *err = NULL;
-        GdkPixbuf *pb;
-		
-        pb = gdk_pixbuf_new_from_file(file, &err);
-        if (err || !pb) {
-            g_error_free(err);
-        } else {
-            GdkPixbuf *pb_scaled;
+    if (!g_file_test(file, G_FILE_TEST_EXISTS)) 
+        goto err;
 
-            pb_scaled = gdk_pixbuf_scale_simple(pb, width, height,
-                  GDK_INTERP_BILINEAR);
-		
-            img = gtk_image_new_from_pixbuf(pb_scaled);			
-		
-            gdk_pixbuf_unref(pb);
-            gdk_pixbuf_unref(pb_scaled);
-
-            RET(img);
-        }
+    if (!(pb = gdk_pixbuf_new_from_file(file, NULL)))
+        goto err;
+    
+    if (keep_ratio) {
+        w = gdk_pixbuf_get_width(pb);
+        h = gdk_pixbuf_get_height(pb);
+        rw = w / width;
+        rh = h / height;
+        if (rw > rh)
+            height = h / rw;
+        else
+            width =  w / rh;
     }
+    pb_scaled = gdk_pixbuf_scale_simple(pb, width, height,
+                                        GDK_INTERP_BILINEAR);
+    img = gtk_image_new_from_pixbuf(pb_scaled);			
+    g_object_unref(pb);
+    g_object_unref(pb_scaled);
+    RET(img);
 
+ err:
     img = gtk_image_new_from_stock(GTK_STOCK_MISSING_IMAGE,
-          GTK_ICON_SIZE_BUTTON);
-
+                                   GTK_ICON_SIZE_BUTTON);
     RET(img);
 }
 
@@ -693,13 +734,14 @@ get_button_spacing(GtkRequisition *req, GtkContainer *parent, gchar *name)
 
     ENTER;
     b = gtk_button_new();
-    if (parent)
-        gtk_container_add(parent, b);
     gtk_widget_set_name(GTK_WIDGET(b), name);
     GTK_WIDGET_UNSET_FLAGS (b, GTK_CAN_FOCUS);
     GTK_WIDGET_UNSET_FLAGS (b, GTK_CAN_DEFAULT);
     gtk_container_set_border_width (GTK_CONTAINER (b), 0);
-   
+
+    if (parent)
+        gtk_container_add(parent, b);
+
     gtk_widget_show(b);
     gtk_widget_size_request(b, req);
 
