@@ -36,10 +36,11 @@ int get_line(FILE *fp, line *s);
 int get_line_as_is(FILE *fp, line *s);
 
 void Xclimsg(Window win, long type, long l0, long l1, long l2, long l3, long l4);
+void Xclimsgwm(Window win, Atom type, Atom arg);
 void *get_xaproperty (Window win, Atom prop, Atom type, int *nitems);
 char *get_textproperty(Window win, Atom prop);
 void resolve_atoms();
-Window Select_Window(Display *dpy);
+//Window Select_Window(Display *dpy);
 int get_net_number_of_desktops();
 int get_net_current_desktop ();
 int get_net_wm_desktop(Window win);
@@ -49,7 +50,10 @@ void get_net_wm_window_type(Window win, net_wm_window_type *nwwt);
 
 void calculate_position(panel *np);
 gchar *expand_tilda(gchar *file);
-GtkWidget *gtk_image_new_from_file_scaled(const gchar *file, gint width, gint height);
+GdkPixbuf *gdk_pixbuf_scale_ratio(GdkPixbuf *p, int width, int height, GdkInterpType itype,
+                                  gboolean keep_ratio);
+GtkWidget *gtk_image_new_from_file_scaled(const gchar *file, gint width, gint height,  
+                                          gboolean keep_ratio);
 void get_button_spacing(GtkRequisition *req, GtkContainer *parent, gchar *name);
 
 #endif
