@@ -14,14 +14,15 @@
 #include "panel.h"
 #include "misc.h"
 #include "bg.h"
-
-/* do not change this line - Makefile's 'tar' target depends on it */
-#define VERSION "3.8"
+#include "version.h"
+#include "gtkbgbox.h"
 
 static gchar *cfgfile = NULL;
 static gchar version[] = VERSION;
 static gchar *cprofile = "default";
 
+int verbose;
+FbEv *fbev;
 
 //#define DEBUG
 #include "dbg.h"
@@ -31,30 +32,8 @@ static gchar *cprofile = "default";
 
 static FILE *pfp; /* current profile FP */
 static panel *p;
-static gchar *transparent_rc = "style 'transparent-style'\n"
-"{\n"
-"bg_pixmap[NORMAL] = \"<parent>\"\n"
-"bg_pixmap[INSENSITIVE] = \"<parent>\"\n"
-"bg_pixmap[PRELIGHT] = \"<parent>\"\n"
-"bg_pixmap[SELECTED] = \"<parent>\"\n"
-"bg_pixmap[ACTIVE] = \"<parent>\"\n"
-"}\n"
-"class \"GtkEventBox\" style \"transparent-style\"\n"
-"class \"GtkSocket\" style \"transparent-style\"\n"
-"class \"GtkBar\" style \"transparent-style\"\n"
-"class \"GtkBox\" style \"transparent-style\"\n"
-"\n";
 
 
-/*
-"class \"GtkBox\" style \"transparent-style\"\n"
-"class \"GtkContainer\" style \"transparent-style\"\n"
-"class \"GtkBin\" style \"transparent-style\"\n"
-"class \"GtkSeparator\" style \"transparent-style\"\n"
-*/
-
-static void set_bg(GtkWidget *widget, panel *p);
-
 /****************************************************
  *         panel's handlers for WM events           *
  ****************************************************/
@@ -67,6 +46,7 @@ panel_del_wm_strut(panel *p)
 }
 */
 
+
 static void
 panel_set_wm_strut(panel *p)
 {
@@ -136,7 +116,7 @@ print_wmdata(panel *p)
 
 
 static GdkFilterReturn
-panel_wm_events(GdkXEvent *xevent, GdkEvent *event, panel *p)
+panel_event_filter(GdkXEvent *xevent, GdkEvent *event, panel *p)
 {
     Atom at;
     Window win;
@@ -149,28 +129,39 @@ panel_wm_events(GdkXEvent *xevent, GdkEvent *event, panel *p)
     
     at = ev->xproperty.atom;
     win = ev->xproperty.window;
+    DBG("win=%x at=%d\n", win, at);
     if (win == GDK_ROOT_WINDOW()) {
 	if (at == a_NET_CLIENT_LIST) {
             DBG("A_NET_CLIENT_LIST\n");
+            fb_ev_trigger(fbev, EV_CLIENT_LIST);
 	} else if (at == a_NET_CURRENT_DESKTOP) {
-            p->curdesk = get_net_current_desktop();
             DBG("A_NET_CURRENT_DESKTOP\n");
+            p->curdesk = get_net_current_desktop();
+            fb_ev_trigger(fbev, EV_CURRENT_DESKTOP);
 	} else if (at == a_NET_NUMBER_OF_DESKTOPS) {
-            p->desknum = get_net_number_of_desktops();
             DBG("A_NET_NUMBER_OF_DESKTOPS\n");
+            p->desknum = get_net_number_of_desktops();
+            fb_ev_trigger(fbev, EV_NUMBER_OF_DESKTOPS);
 	} else if (at == a_NET_ACTIVE_WINDOW) {
             DBG("A_NET_ACTIVE_WINDOW\n");
+            fb_ev_trigger(fbev, EV_ACTIVE_WINDOW);
+        } else if (at == a_NET_CLIENT_LIST_STACKING) {
+            DBG("A_NET_CLIENT_LIST_STACKING\n");
+            fb_ev_trigger(fbev, EV_CLIENT_LIST_STACKING);
         } else if (at == a_XROOTPMAP_ID) {
-            bg_rootbg_changed();
-            set_bg(p->topgwin, p);
-            gtk_widget_queue_draw(p->topgwin);
             DBG("a_XROOTPMAP_ID\n");
+            if (p->transparent) {
+                fb_bg_notify_changed_bg(p->bg);
+            }
 	} else if (at == a_NET_WORKAREA) {
             DBG("A_NET_WORKAREA\n");
             p->workarea = get_xaproperty (GDK_ROOT_WINDOW(), a_NET_WORKAREA, XA_CARDINAL, &p->wa_len);
             print_wmdata(p);
-        }
-    } 
+        } else
+            RET(GDK_FILTER_CONTINUE);
+        RET(GDK_FILTER_REMOVE);
+    }
+    DBG("non root %x\n", win);
     RET(GDK_FILTER_CONTINUE);
 }
 
@@ -299,63 +290,6 @@ make_round_corners(panel *p)
     RET();      
 }
 
-static void
-set_bg(GtkWidget *widget, panel *p)
-{
-    ENTER;
-    /*
-    if (p->xtopbg != None)
-        XFreePixmap(GDK_DISPLAY(), p->xtopbg);
-    p->xtopbg = bg_new_for_win(p->topxwin);
-    if (p->gtopbg)
-        g_object_unref(G_OBJECT(p->gtopbg));
-    p->gtopbg = gdk_xid_table_lookup (p->xtopbg);
-    if (p->gtopbg)
-        g_object_ref (G_OBJECT (p->gtopbg));
-    else
-        p->gtopbg = gdk_pixmap_foreign_new (p->xtopbg);
-    */
-    if (p->gtopbg) 
-        g_object_unref(p->gtopbg);
-    p->gtopbg = bg_new_for_win(p->topxwin);
-
-    
-    modify_drawable(p->gtopbg, p->topgwin->style->black_gc, p->tintcolor, p->alpha);
-    
-    gdk_window_set_back_pixmap(p->topgwin->window, p->gtopbg, FALSE);
-    gdk_window_clear(p->topgwin->window);
-    gtk_widget_queue_draw_area (p->topgwin, 0, 0, 2000, 2000);
-    RET();
-}
-
-static void
-panel_style_set(GtkWidget *widget, GtkStyle *s, panel *p)
-{
-    ENTER;
-    
-    gtk_rc_parse_string(transparent_rc);
-    if (GTK_WIDGET_REALIZED(widget)) 
-        set_bg(widget, p);
-    RET();
-}
-
-static gboolean
-panel_configure_event(GtkWidget *widget, GdkEventConfigure *event, panel *p)
-{
-    static gint x = 0, y = 0, width = 0, height = 0;
-
-    ENTER;
-    if (x == event->x && y == event->y
-          && width == event->width && height == event->height)
-        RET(FALSE);
-    x = event->x;
-    y = event->y;
-    width = event->width;
-    height = event->height;
-    set_bg(widget, p);
-    RET(FALSE);
-}
-
 void
 panel_start_gui(panel *p)
 {
@@ -365,7 +299,8 @@ panel_start_gui(panel *p)
  
     
     ENTER;
-    //gtk_rc_parse_string(transparent_rc);
+
+    // main toplevel window
     p->topgwin =  gtk_window_new(GTK_WINDOW_TOPLEVEL);
     gtk_container_set_border_width(GTK_CONTAINER(p->topgwin), 0);
     gtk_window_set_resizable(GTK_WINDOW(p->topgwin), FALSE);
@@ -379,25 +314,28 @@ panel_start_gui(panel *p)
           (GCallback) panel_size_req, p);
     g_signal_connect (G_OBJECT (p->topgwin), "size-allocate",
           (GCallback) panel_size_alloc, p);
-    
-    if (p->transparent) {
-        g_signal_connect (G_OBJECT (p->topgwin), "configure-event",
-              (GCallback) panel_configure_event, p);
-
-        g_signal_connect (G_OBJECT (p->topgwin), "style-set",
-              (GCallback) panel_style_set, p);
-    }
     gtk_widget_realize(p->topgwin);
     gdk_window_set_decorations(p->topgwin->window, 0);
     gtk_widget_set_app_paintable(p->topgwin, TRUE);
-    
+
+    // background box all over toplevel
+    p->bbox = gtk_bgbox_new();
+    gtk_container_add(GTK_CONTAINER(p->topgwin), p->bbox);
+    gtk_widget_show(p->bbox);
+    gtk_container_set_border_width(GTK_CONTAINER(p->bbox), 0);
+    if (p->transparent) {
+        p->bg = fb_bg_get_for_display();
+        gtk_bgbox_set_background(p->bbox, BG_ROOT, p->tintcolor, p->alpha);        
+    }
+
+    // main layout manager as a single child of background widget box
     p->lbox = p->my_box_new(FALSE, 0);
     gtk_container_set_border_width(GTK_CONTAINER(p->lbox), 0);
-    gtk_container_add(GTK_CONTAINER(p->topgwin), p->lbox);
+    gtk_container_add(GTK_CONTAINER(p->bbox), p->lbox);
     gtk_widget_show(p->lbox);
     if (p->round_corners)
         make_round_corners(p);
-
+ 
     p->box = p->my_box_new(FALSE, 1); 
     gtk_container_set_border_width(GTK_CONTAINER(p->box), 1);
     gtk_box_pack_start(GTK_BOX(p->lbox), p->box, TRUE, TRUE, 0);
@@ -405,53 +343,48 @@ panel_start_gui(panel *p)
       
     p->topxwin = GDK_WINDOW_XWINDOW(GTK_WIDGET(p->topgwin)->window);
     
-    bg_init(GDK_DISPLAY());
-    
-    /* make our window unfocusable */
+
+    /* the settings that should be done before window is mapped */
     wmhints.flags = InputHint;
     wmhints.input = 0;
     XSetWMHints (GDK_DISPLAY(), p->topxwin, &wmhints); 
-    if (p->setdocktype) {
-        state[0] = a_NET_WM_WINDOW_TYPE_DOCK;
-        XChangeProperty(GDK_DISPLAY(), p->topxwin, a_NET_WM_WINDOW_TYPE, XA_ATOM,
-              32, PropModeReplace, (unsigned char *) state, 1);
-    }
-
-    
-    
 #define WIN_HINTS_SKIP_FOCUS      (1<<0)	/* "alt-tab" skips this win */
     val = WIN_HINTS_SKIP_FOCUS;
     XChangeProperty(GDK_DISPLAY(), p->topxwin,
           XInternAtom(GDK_DISPLAY(), "_WIN_HINTS", False), XA_CARDINAL, 32,
           PropModeReplace, (unsigned char *) &val, 1);
 
-    Xclimsg(p->topxwin, a_NET_WM_DESKTOP, 0xFFFFFFFF, 0, 0, 0, 0);
+    if (p->setdocktype) {
+        state[0] = a_NET_WM_WINDOW_TYPE_DOCK;
+        XChangeProperty(GDK_DISPLAY(), p->topxwin, a_NET_WM_WINDOW_TYPE, XA_ATOM,
+              32, PropModeReplace, (unsigned char *) state, 1);
+    }
 
-    /************************/
-    /* Window Mapping Point */
+    /* window mapping point */
     gtk_widget_show_all(p->topgwin);
+
+    /* the settings that should be done after window is mapped */
+
+    /* send it to running wm */
     Xclimsg(p->topxwin, a_NET_WM_DESKTOP, 0xFFFFFFFF, 0, 0, 0, 0);
-    
+    /* and assign it ourself just for case when wm is not running */
+    val = 0xFFFFFFFF;
+    XChangeProperty(GDK_DISPLAY(), p->topxwin, a_NET_WM_DESKTOP, XA_CARDINAL, 32,
+          PropModeReplace, (unsigned char *) &val, 1);
+
     state[0] = a_NET_WM_STATE_SKIP_PAGER;
     state[1] = a_NET_WM_STATE_SKIP_TASKBAR;
     state[2] = a_NET_WM_STATE_STICKY;
     XChangeProperty(GDK_DISPLAY(), p->topxwin, a_NET_WM_STATE, XA_ATOM,
           32, PropModeReplace, (unsigned char *) state, 3);
 
- 
-  
     XSelectInput (GDK_DISPLAY(), GDK_ROOT_WINDOW(), PropertyChangeMask);
-    /*
-      XSelectInput (GDK_DISPLAY(), topxwin, PropertyChangeMask | FocusChangeMask |
-      StructureNotifyMask);
-    */
-    gdk_window_add_filter(gdk_get_default_root_window (), (GdkFilterFunc)panel_wm_events, p);
+    gdk_window_add_filter(gdk_get_default_root_window (), (GdkFilterFunc)panel_event_filter, p);
 
     calculate_position(p);
     gdk_window_move_resize(p->topgwin->window, p->ax, p->ay, p->aw, p->ah);
     if (p->setstrut)
         panel_set_wm_strut(p);
-
   
     RET();
 }
@@ -547,13 +480,15 @@ panel_parse_plugin(panel *p, FILE *fp)
     }
     expand = padding = 0;
     while (get_line(fp, &s) != LINE_BLOCK_END) {
-        if (s.type == LINE_NONE)
+        if (s.type == LINE_NONE) {
+            ERR( "fbpanel: bad line %s\n", s.str);
             goto error;
-
+        }
         if (s.type == LINE_VAR) {
-            if (!g_ascii_strcasecmp(s.t[0], "type"))
+            if (!g_ascii_strcasecmp(s.t[0], "type")) {
                 type = g_strdup(s.t[1]);
-            else if (!g_ascii_strcasecmp(s.t[0], "expand"))
+                DBG("plug %s\n", type);
+            } else if (!g_ascii_strcasecmp(s.t[0], "expand"))
                 expand = str2num(bool_pair,  s.t[1], 0);
             else if (!g_ascii_strcasecmp(s.t[0], "padding"))
                 padding = atoi(s.t[1]);
@@ -596,6 +531,7 @@ panel_parse_plugin(panel *p, FILE *fp)
     plug->padding = padding;
     fprintf(tmpfp, "}\n");
     fseek(tmpfp, 0, SEEK_SET);
+    DBG("starting\n");
     if (!plugin_start(plug)) {
         ERR( "fbpanel: can't start plugin %s\n", type);
         goto error;
@@ -635,7 +571,7 @@ panel_start(panel *p, FILE *fp)
     p->transparent = 0;
     p->alpha = 127;
     p->tintcolor = 0xFFFFFFFF;
-    p->xtopbg = None;
+    fbev = fb_ev_new();
     if ((get_line(fp, &s) != LINE_BLOCK_START) || g_ascii_strcasecmp(s.t[0], "Global")) {
         ERR( "fbpanel: config file must start from Global section\n");
         RET(0);
@@ -675,8 +611,9 @@ void panel_stop(panel *p)
     g_list_foreach(p->plugins, delete_plugin, NULL);
     g_list_free(p->plugins);
     p->plugins = NULL;
+    g_object_unref(fbev);
     XSelectInput (GDK_DISPLAY(), GDK_ROOT_WINDOW(), NoEventMask);
-    gdk_window_remove_filter(gdk_get_default_root_window (), (GdkFilterFunc)panel_wm_events, p);
+    gdk_window_remove_filter(gdk_get_default_root_window (), (GdkFilterFunc)panel_event_filter, p);
     gtk_widget_destroy(p->topgwin);
     g_free(p->workarea);
     RET();
@@ -689,8 +626,9 @@ usage()
     ENTER;
     printf("fbpanel %s - lightweight GTK2+ panel for UNIX desktops\n", version);
     printf("Command line options:\n");
-    printf(" -h  -- print this help and exit:\n");
-    printf(" -v  -- print version and exit:\n");
+    printf(" -h  -- print this help and exit\n");
+    printf(" -v  -- print version and exit\n");
+    printf(" -verbose  -- print usefull messages at runtime\n");
     printf(" -p <name> -- use named profile. File ~/.fbpanel/<name> must exist\n");
     printf("\nVisit http://fbpanel.sourceforge.net/ for detailed documentation,\n");
     printf("sample profiles and other stuff.\n\n");
@@ -706,20 +644,22 @@ open_profile(gchar *profile)
     fname = g_strdup_printf("%s/.fbpanel/%s", getenv("HOME"), profile);
     if ((fp = fopen(fname, "r"))) {
         cfgfile = fname;
-        ERR("Using %s\n", fname);
+        if (verbose)
+            ERR("Using %s\n", fname);
         RET(fp);
     }
-    ERR("Can't load %s\n", fname);
+    //ERR("Can't load %s\n", fname);
     g_free(fname);
     
     /* check private configuration directory */
     fname = g_strdup_printf("%s/share/fbpanel/%s", PREFIX, profile);
     if ((fp = fopen(fname, "r"))) {
         cfgfile = fname;
-        ERR("Using %s\n", fname);
+        if (verbose)
+            ERR("Using %s\n", fname);
         RET(fp);
     }
-    ERR("Can't load %s\n", fname);
+    //ERR("Can't load %s\n", fname);
     g_free(fname);
     ERR("Can't open '%s' profile\n", profile);
     RET(NULL);
@@ -731,8 +671,10 @@ handle_error(Display * d, XErrorEvent * ev)
     char buf[256];
 
     ENTER;
-    XGetErrorText(GDK_DISPLAY(), ev->error_code, buf, 256);
-    ERR( "fbpanel : X error: %s\n", buf);
+    if (verbose) {
+        XGetErrorText(GDK_DISPLAY(), ev->error_code, buf, 256);
+        ERR( "fbpanel : X error: %s\n", buf);
+    }
     RET();
 }
 
@@ -764,6 +706,8 @@ main(int argc, char *argv[], char *env[])
         } else if (!strcmp(argv[i], "-v")) {
             printf("fbpanel %s\n", version);
             exit(0);
+        } else if (!strcmp(argv[i], "-verbose")) {
+            verbose = 1;
         } else if (!strcmp(argv[i], "-p")) {
             i++;
             if (i == argc) {
